^ expected errors | v input
// Copyright (c) 2017 Mike Pennisi.  All rights reserved.
// This code is governed by the license found in the LICENSE file.
/*---
esid: sec-assignment-operators-static-semantics-early-errors
description: Minimal test
---*/

void 0;
