^ expected errors | v input
// Copyright 2017 Mike Pennisi.  All rights reserved.
// See LICENSE for details.
/*---
esid: sec-assignment-operators-static-semantics-early-errors
description: Minimal test
---*/

void 0;
